@@ -7,20 +7,19 @@ import (
 	"sync"
 	"time"
 
-	"github.com/gdamore/tcell/v2"
-
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/hooks"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
 	"git.sr.ht/~rjarry/aerc/lib/state"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"git.sr.ht/~rockorager/vaxis"
 )
 
 var _ ProvidesMessages = (*AccountView)(nil)
@@ -41,7 +40,8 @@ type AccountView struct {
 	split         *MessageViewer
 	splitSize     int
 	splitDebounce *time.Timer
-	splitDir      string
+	splitDir      config.SplitDirection
+	splitLoaded   bool
 
 	// Check-mail ticker
 	ticker       *time.Ticker
@@ -168,7 +168,7 @@ func (acct *AccountView) Draw(ctx *ui.Context) {
 	acct.grid.Draw(ctx)
 }
 
-func (acct *AccountView) MouseEvent(localX int, localY int, event tcell.Event) {
+func (acct *AccountView) MouseEvent(localX int, localY int, event vaxis.Event) {
 	acct.grid.MouseEvent(localX, localY, event)
 }
 
@@ -242,18 +242,28 @@ func (acct *AccountView) isSelected() bool {
 
 func (acct *AccountView) newStore(name string) *lib.MessageStore {
 	uiConf := acct.dirlist.UiConfig(name)
+	dir := acct.dirlist.Directory(name)
+	role := ""
+	if dir != nil {
+		role = string(dir.Role)
+	}
+	backend := acct.AccountConfig().Backend
 	store := lib.NewMessageStore(acct.worker,
 		acct.sortCriteria(uiConf),
 		uiConf.ThreadingEnabled,
 		uiConf.ForceClientThreads,
 		uiConf.ClientThreadsDelay,
+		uiConf.SelectLast,
+		uiConf.ThreadingBySubject,
 		uiConf.ReverseOrder,
 		uiConf.ReverseThreadOrder,
 		uiConf.SortThreadSiblings,
 		func(msg *models.MessageInfo) {
 			err := hooks.RunHook(&hooks.MailReceived{
 				Account: acct.Name(),
+				Backend: backend,
 				Folder:  name,
+				Role:    role,
 				MsgInfo: msg,
 			})
 			if err != nil {
@@ -267,7 +277,9 @@ func (acct *AccountView) newStore(name string) *lib.MessageStore {
 		}, func() {
 			err := hooks.RunHook(&hooks.MailDeleted{
 				Account: acct.Name(),
+				Backend: backend,
 				Folder:  name,
+				Role:    role,
 			})
 			if err != nil {
 				msg := fmt.Sprintf("mail-deleted hook: %s", err)
@@ -276,12 +288,37 @@ func (acct *AccountView) newStore(name string) *lib.MessageStore {
 		}, func(dest string) {
 			err := hooks.RunHook(&hooks.MailAdded{
 				Account: acct.Name(),
+				Backend: backend,
 				Folder:  dest,
+				Role:    role,
 			})
 			if err != nil {
 				msg := fmt.Sprintf("mail-added hook: %s", err)
 				PushError(msg)
 			}
+		}, func(add []string, remove []string) {
+			err := hooks.RunHook(&hooks.TagModified{
+				Account: acct.Name(),
+				Backend: backend,
+				Add:     add,
+				Remove:  remove,
+			})
+			if err != nil {
+				msg := fmt.Sprintf("tag-modified hook: %s", err)
+				PushError(msg)
+			}
+		}, func(flagname string) {
+			err := hooks.RunHook(&hooks.FlagChanged{
+				Account:  acct.Name(),
+				Backend:  backend,
+				Folder:   acct.SelectedDirectory(),
+				Role:     role,
+				FlagName: flagname,
+			})
+			if err != nil {
+				msg := fmt.Sprintf("flag-changed hook: %s", err)
+				PushError(msg)
+			}
 		},
 		acct.updateSplitView,
 		acct.dirlist.UiConfig(name).ThreadContext,
@@ -545,7 +582,7 @@ func (acct *AccountView) closeSplit() {
 		acct.split.Close()
 	}
 	acct.splitSize = 0
-	acct.splitDir = ""
+	acct.splitDir = config.SPLIT_NONE
 	acct.split = nil
 	acct.grid = ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
@@ -562,7 +599,16 @@ func (acct *AccountView) closeSplit() {
 }
 
 func (acct *AccountView) updateSplitView(msg *models.MessageInfo) {
-	if acct.splitSize == 0 {
+	if !acct.splitLoaded {
+		switch acct.uiConf.MessageListSplit.Direction {
+		case config.SPLIT_HORIZONTAL:
+			acct.Split(acct.uiConf.MessageListSplit.Size)
+		case config.SPLIT_VERTICAL:
+			acct.Vsplit(acct.uiConf.MessageListSplit.Size)
+		}
+		acct.splitLoaded = true
+	}
+	if acct.splitSize == 0 || !acct.splitLoaded {
 		return
 	}
 	if acct.splitDebounce != nil {
@@ -581,9 +627,9 @@ func (acct *AccountView) updateSplitView(msg *models.MessageInfo) {
 				}
 				acct.split = NewMessageViewer(acct, view)
 				switch acct.splitDir {
-				case "split", "hsplit":
+				case config.SPLIT_HORIZONTAL:
 					acct.grid.AddChild(acct.split).At(1, 1)
-				case "vsplit":
+				case config.SPLIT_VERTICAL:
 					acct.grid.AddChild(acct.split).At(0, 2)
 				}
 			})
@@ -606,12 +652,12 @@ func (acct *AccountView) SetSplitSize(n int) {
 
 // Split splits the message list view horizontally. The message list will be n
 // rows high. If n is 0, any existing split is removed
-func (acct *AccountView) Split(n int) error {
+func (acct *AccountView) Split(n int) {
 	acct.SetSplitSize(n)
-	if acct.splitDir == "split" || n == 0 {
-		return nil
+	if acct.splitDir == config.SPLIT_HORIZONTAL || n == 0 {
+		return
 	}
-	acct.splitDir = "split"
+	acct.splitDir = config.SPLIT_HORIZONTAL
 	acct.grid = ui.NewGrid().Rows([]ui.GridSpec{
 		// Add 1 so that the splitSize is the number of visible messages
 		{Strategy: ui.SIZE_EXACT, Size: func() int { return acct.SplitSize() + 1 }},
@@ -628,17 +674,16 @@ func (acct *AccountView) Split(n int) error {
 	acct.split = NewMessageViewer(acct, nil)
 	acct.grid.AddChild(acct.split).At(1, 1)
 	acct.updateSplitView(acct.msglist.Selected())
-	return nil
 }
 
 // Vsplit splits the message list view vertically. The message list will be n
 // rows wide. If n is 0, any existing split is removed
-func (acct *AccountView) Vsplit(n int) error {
+func (acct *AccountView) Vsplit(n int) {
 	acct.SetSplitSize(n)
-	if acct.splitDir == "vsplit" || n == 0 {
-		return nil
+	if acct.splitDir == config.SPLIT_VERTICAL || n == 0 {
+		return
 	}
-	acct.splitDir = "vsplit"
+	acct.splitDir = config.SPLIT_VERTICAL
 	acct.grid = ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
 	}).Columns([]ui.GridSpec{
@@ -654,7 +699,6 @@ func (acct *AccountView) Vsplit(n int) error {
 	acct.split = NewMessageViewer(acct, nil)
 	acct.grid.AddChild(acct.split).At(0, 2)
 	acct.updateSplitView(acct.msglist.Selected())
-	return nil
 }
 
 // setTitle executes the title template and sets the tab title
