@@ -5,10 +5,10 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"math/rand"
 	"net/textproto"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"sort"
 	"strconv"
 	"strings"
@@ -17,20 +17,23 @@ import (
 	"time"
 
 	"github.com/emersion/go-message/mail"
-	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
 	"github.com/pkg/errors"
 
+	"git.sr.ht/~rjarry/aerc/commands/mode"
 	"git.sr.ht/~rjarry/aerc/completer"
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/log"
+	"git.sr.ht/~rjarry/aerc/lib/send"
 	"git.sr.ht/~rjarry/aerc/lib/state"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/xdg"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
+	"git.sr.ht/~rockorager/vaxis"
 )
 
 type Composer struct {
@@ -120,6 +123,8 @@ func NewComposer(
 		return nil, err
 	}
 
+	mode.NoQuit()
+
 	return c, nil
 }
 
@@ -723,6 +728,7 @@ func (c *Composer) Close() {
 		c.editor.Destroy()
 		c.editor = nil
 	}
+	mode.NoQuitDone()
 }
 
 func (c *Composer) Bindings() string {
@@ -751,7 +757,7 @@ func (c *Composer) focusActiveWidget(focus bool) {
 	}
 }
 
-func (c *Composer) Event(event tcell.Event) bool {
+func (c *Composer) Event(event vaxis.Event) bool {
 	c.Lock()
 	defer c.Unlock()
 	if w := c.focusedWidget(); c.editor != nil && w != nil {
@@ -760,7 +766,7 @@ func (c *Composer) Event(event tcell.Event) bool {
 	return false
 }
 
-func (c *Composer) MouseEvent(localX int, localY int, event tcell.Event) {
+func (c *Composer) MouseEvent(localX int, localY int, event vaxis.Event) {
 	c.Lock()
 	for _, e := range c.focusable {
 		he, ok := e.(*headerEditor)
@@ -820,7 +826,15 @@ func (c *Composer) PrepareHeader() (*mail.Header, error) {
 	// control headers not normally set by the user
 	// repeated calls to PrepareHeader should be a noop
 	if !c.header.Has("Message-Id") {
-		hostname, err := getMessageIdHostname(c)
+		froms, err := c.header.AddressList("from")
+		if err != nil {
+			return nil, err
+		}
+		if len(froms) == 0 {
+			return nil, fmt.Errorf("no valid From address found")
+		}
+		hostname, err := send.GetMessageIdHostname(
+			c.acctConfig.SendWithHostname, froms[0])
 		if err != nil {
 			return nil, err
 		}
@@ -839,25 +853,6 @@ func (c *Composer) PrepareHeader() (*mail.Header, error) {
 	return c.header, nil
 }
 
-func getMessageIdHostname(c *Composer) (string, error) {
-	if c.acctConfig.SendWithHostname {
-		return os.Hostname()
-	}
-	addrs, err := c.header.AddressList("from")
-	if err != nil {
-		return "", err
-	}
-	if len(addrs) == 0 {
-		// no from address present, generate a random hostname
-		return strings.ToUpper(strconv.FormatInt(rand.Int63(), 36)), nil
-	}
-	_, domain, found := strings.Cut(addrs[0].Address, "@")
-	if !found {
-		return "", fmt.Errorf("Invalid address %q", addrs[0])
-	}
-	return domain, nil
-}
-
 func (c *Composer) parseEmbeddedHeader() (*mail.Header, error) {
 	_, err := c.email.Seek(0, io.SeekStart)
 	if err != nil {
@@ -953,19 +948,20 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 			}
 		}
 
-		if header != nil && !header.Has("MIME-Version") {
-			// sign and encrypt will create multipart/* messages
-			// without setting the MIME-Version header. Set it
-			// manually at the top level to be compliant with RFC
-			// 2045.
-			header.Set("MIME-Version", "1.0")
-		}
-
 		if c.encrypt {
 			rcpts, err := getRecipientsEmail(c)
 			if err != nil {
 				return err
 			}
+
+			if c.acct.acct.PgpSelfEncrypt {
+				signer, err := c.Signer()
+				if err != nil {
+					return err
+				}
+				rcpts = append(rcpts, signer)
+			}
+
 			cleartext, err = CryptoProvider().Encrypt(&buf, rcpts, signer, DecryptKeys, header)
 			if err != nil {
 				return err
@@ -1022,6 +1018,22 @@ func (c *Composer) ShouldWarnSubject() bool {
 	return len(subject) == 0
 }
 
+func (c *Composer) CheckForMultipartErrors() error {
+	problems := []string{}
+	for _, p := range c.textParts {
+		if p.ConversionError != nil {
+			text := fmt.Sprintf("%s: %s", p.MimeType, p.ConversionError.Error())
+			problems = append(problems, text)
+		}
+	}
+
+	if len(problems) == 0 {
+		return nil
+	}
+
+	return fmt.Errorf("multipart conversion error: %s", strings.Join(problems, "; "))
+}
+
 func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
 	mimeParams := map[string]string{"Charset": "UTF-8"}
 	if config.Compose.FormatFlowed {
@@ -1109,6 +1121,8 @@ func (c *Composer) GetAttachments() []string {
 }
 
 func (c *Composer) AddAttachment(path string) {
+	path, _ = filepath.Abs(path)
+	path = xdg.TildeHome(path)
 	c.attachments = append(c.attachments, lib.NewFileAttachment(path))
 	c.resetReview()
 }
@@ -1147,9 +1161,9 @@ func (c *Composer) resetReview() {
 	}
 }
 
-func (c *Composer) termEvent(event tcell.Event) bool {
-	if event, ok := event.(*tcell.EventMouse); ok {
-		if event.Buttons() == tcell.Button1 {
+func (c *Composer) termEvent(event vaxis.Event) bool {
+	if event, ok := event.(vaxis.Mouse); ok {
+		if event.Button == vaxis.MouseLeftButton {
 			c.FocusTerminal()
 			return true
 		}
@@ -1566,9 +1580,9 @@ func (he *headerEditor) Draw(ctx *ui.Context) {
 	he.input.Draw(ctx.Subcontext(size, 0, ctx.Width()-size, 1))
 }
 
-func (he *headerEditor) MouseEvent(localX int, localY int, event tcell.Event) {
-	if event, ok := event.(*tcell.EventMouse); ok {
-		if event.Buttons() == tcell.Button1 {
+func (he *headerEditor) MouseEvent(localX int, localY int, event vaxis.Event) {
+	if event, ok := event.(vaxis.Mouse); ok {
+		if event.Button == vaxis.MouseLeftButton {
 			he.focused = true
 		}
 
@@ -1588,7 +1602,7 @@ func (he *headerEditor) Focus(focused bool) {
 	he.input.Focus(focused)
 }
 
-func (he *headerEditor) Event(event tcell.Event) bool {
+func (he *headerEditor) Event(event vaxis.Event) bool {
 	return he.input.Event(event)
 }
 
@@ -1615,7 +1629,14 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 	)
 	bindings = bindings.ForFolder(composer.SelectedDirectory())
 
-	reviewCommands := [][]string{
+	const maxInputWidth = 6
+	type reviewCmd struct {
+		output     string
+		annotation string
+		input      string
+	}
+
+	reviewCmds := []reviewCmd{
 		{":send<enter>", "Send", ""},
 		{":edit<enter>", "Edit", ""},
 		{":attach<space>", "Add attachment", ""},
@@ -1625,50 +1646,59 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 		{":abort<enter>", "Abort (discard message, no confirmation)", ""},
 		{":choose -o d discard abort -o p postpone postpone<enter>", "Abort or postpone", ""},
 	}
-	knownCommands := len(reviewCommands)
+	knownCommands := len(reviewCmds)
 	var actions []string
 	for _, binding := range bindings.Bindings {
 		inputs := config.FormatKeyStrokes(binding.Input)
 		outputs := config.FormatKeyStrokes(binding.Output)
 		found := false
-		for i, rcmd := range reviewCommands {
-			if outputs == rcmd[0] {
+		for i, rcmd := range reviewCmds {
+			if outputs == rcmd.output {
 				found = true
-				if reviewCommands[i][2] == "" {
-					reviewCommands[i][2] = inputs
+				if reviewCmds[i].input == "" {
+					reviewCmds[i].input = inputs
 				} else {
-					reviewCommands[i][2] += ", " + inputs
+					reviewCmds[i].input += ", " + inputs
+				}
+				if binding.Annotation != "" {
+					// overwrite default description with
+					// user annotations if present
+					reviewCmds[i].annotation = binding.Annotation
 				}
 				break
 			}
 		}
 		if !found {
-			rcmd := []string{outputs, "", inputs}
-			reviewCommands = append(reviewCommands, rcmd)
+			rcmd := reviewCmd{
+				output:     outputs,
+				annotation: binding.Annotation,
+				input:      inputs,
+			}
+			reviewCmds = append(reviewCmds, rcmd)
 		}
 	}
-	unknownCommands := reviewCommands[knownCommands:]
+	unknownCommands := reviewCmds[knownCommands:]
 	sort.Slice(unknownCommands, func(i, j int) bool {
-		return unknownCommands[i][2] < unknownCommands[j][2]
+		return unknownCommands[i].input < unknownCommands[j].input
 	})
 
 	longest := 0
-	for _, rcmd := range reviewCommands {
-		if len(rcmd[2]) > longest {
-			longest = len(rcmd[2])
+	for _, rcmd := range reviewCmds {
+		if len(rcmd.input) > longest {
+			longest = len(rcmd.input)
 		}
 	}
 
 	width := longest
-	if longest < 6 {
-		width = 6
+	if longest < maxInputWidth {
+		width = maxInputWidth
 	}
 	widthstr := strconv.Itoa(width)
 
-	for _, rcmd := range reviewCommands {
-		if rcmd[2] != "" {
+	for _, rcmd := range reviewCmds {
+		if rcmd.input != "" {
 			actions = append(actions, fmt.Sprintf("  %-"+widthstr+"s  %-40s  %s",
-				rcmd[2], rcmd[1], rcmd[0]))
+				rcmd.input, rcmd.annotation, rcmd.output))
 		}
 	}
 
@@ -1755,16 +1785,26 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 }
 
 func (c *Composer) updateMultipart(p *lib.Part) error {
+	// conversion errors handling
+	p.ConversionError = nil
+	setError := func(e error) error {
+		p.ConversionError = e
+		return e
+	}
+	if !p.Converted {
+		// text/* multipart created without a command (e.g. by :accept)
+		return nil
+	}
 	command, found := config.Converters[p.MimeType]
 	if !found {
 		// unreachable
-		return fmt.Errorf("no command defined for mime/type")
+		return setError(fmt.Errorf("no command defined for mime/type"))
 	}
 	// reset part body to avoid it leaving outdated if the command fails
 	p.Data = nil
 	body, err := c.GetBody()
 	if err != nil {
-		return errors.Wrap(err, "GetBody")
+		return setError(errors.Wrap(err, "GetBody"))
 	}
 	cmd := exec.Command("sh", "-c", command)
 	cmd.Stdin = body
@@ -1780,7 +1820,7 @@ func (c *Composer) updateMultipart(p *lib.Part) error {
 				stderr = fmt.Sprintf(": %.30s", stderr)
 			}
 		}
-		return fmt.Errorf("%s: %w%s", command, err, stderr)
+		return setError(fmt.Errorf("%s: %w%s", command, err, stderr))
 	}
 	p.Data = out
 	return nil
