@@ -19,7 +19,7 @@ func init() {
 }
 
 func (MakeDir) Context() commands.CommandContext {
-	return commands.ACCOUNT
+	return commands.MESSAGE_LIST
 }
 
 func (MakeDir) Aliases() []string {
@@ -45,15 +45,13 @@ func (m MakeDir) Execute(args []string) error {
 	if acct == nil {
 		return errors.New("No account selected")
 	}
-	previous := acct.SelectedDirectory()
 	acct.Worker().PostAction(&types.CreateDirectory{
 		Directory: m.Folder,
 	}, func(msg types.WorkerMessage) {
 		switch msg := msg.(type) {
 		case *types.Done:
 			app.PushStatus("Directory created.", 10*time.Second)
-			history[acct.Name()] = previous
-			acct.Directories().Open(m.Folder, 0, nil)
+			acct.Directories().Open(m.Folder, "", 0, nil, false)
 		case *types.Error:
 			app.PushError(msg.Error.Error())
 		}
