@@ -18,7 +18,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/format"
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 	"github.com/emersion/go-message/mail"
@@ -30,7 +30,7 @@ type forward struct {
 	Edit       bool     `opt:"-e"`
 	NoEdit     bool     `opt:"-E"`
 	Template   string   `opt:"-T" complete:"CompleteTemplate"`
-	To         []string `opt:"..." required:"false"`
+	To         []string `opt:"..." required:"false" complete:"CompleteTo"`
 }
 
 func init() {
@@ -38,7 +38,7 @@ func init() {
 }
 
 func (forward) Context() commands.CommandContext {
-	return commands.MESSAGE
+	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
 
 func (forward) Aliases() []string {
@@ -49,6 +49,10 @@ func (*forward) CompleteTemplate(arg string) []string {
 	return commands.GetTemplates(arg)
 }
 
+func (*forward) CompleteTo(arg string) []string {
+	return commands.GetAddress(arg)
+}
+
 func (f forward) Execute(args []string) error {
 	if f.AttachAll && f.AttachFull {
 		return errors.New("Options -A and -F are mutually exclusive")
@@ -60,10 +64,6 @@ func (f forward) Execute(args []string) error {
 	if acct == nil {
 		return errors.New("No account selected")
 	}
-	store := widget.Store()
-	if store == nil {
-		return errors.New("Cannot perform action. Messages still loading")
-	}
 	msg, err := widget.SelectedMessage()
 	if err != nil {
 		return err
@@ -110,6 +110,13 @@ func (f forward) Execute(args []string) error {
 		return composer, nil
 	}
 
+	mv, isMsgViewer := widget.(*app.MessageViewer)
+	store := widget.Store()
+	noStore := store == nil
+	if noStore && !isMsgViewer {
+		return errors.New("Cannot perform action. Messages still loading")
+	}
+
 	if f.AttachFull {
 		tmpDir, err := os.MkdirTemp("", "aerc-tmp-attachment")
 		if err != nil {
@@ -117,7 +124,23 @@ func (f forward) Execute(args []string) error {
 		}
 		tmpFileName := path.Join(tmpDir,
 			strings.ReplaceAll(fmt.Sprintf("%s.eml", msg.Envelope.Subject), "/", "-"))
-		store.FetchFull([]uint32{msg.Uid}, func(fm *types.FullMessage) {
+
+		var fetchFull func(func(io.Reader))
+
+		if isMsgViewer {
+			fetchFull = mv.MessageView().FetchFull
+		} else {
+			fetchFull = func(cb func(io.Reader)) {
+				store.FetchFull([]uint32{msg.Uid}, func(fm *types.FullMessage) {
+					if fm == nil || (fm != nil && fm.Content == nil) {
+						return
+					}
+					cb(fm.Content.Reader)
+				})
+			}
+		}
+
+		fetchFull(func(r io.Reader) {
 			tmpFile, err := os.Create(tmpFileName)
 			if err != nil {
 				log.Warnf("failed to create temporary attachment: %v", err)
@@ -129,7 +152,7 @@ func (f forward) Execute(args []string) error {
 			}
 
 			defer tmpFile.Close()
-			_, err = io.Copy(tmpFile, fm.Content.Reader)
+			_, err = io.Copy(tmpFile, r)
 			if err != nil {
 				log.Warnf("failed to write to tmpfile: %v", err)
 				return
@@ -150,7 +173,6 @@ func (f forward) Execute(args []string) error {
 
 		var fetchBodyPart func([]int, func(io.Reader))
 
-		mv, isMsgViewer := widget.(*app.MessageViewer)
 		if isMsgViewer {
 			fetchBodyPart = mv.MessageView().FetchBodyPart
 		} else {
