@@ -13,7 +13,7 @@ func init() {
 }
 
 func (ToggleThreads) Context() commands.CommandContext {
-	return commands.MESSAGE
+	return commands.MESSAGE_LIST | commands.MESSAGE_VIEWER
 }
 
 func (ToggleThreads) Aliases() []string {
