@@ -9,9 +9,11 @@ import (
 	"path"
 	"regexp"
 	"strings"
+	"unicode"
+	"unicode/utf8"
 
-	"git.sr.ht/~rjarry/aerc/log"
-	"github.com/gdamore/tcell/v2"
+	"git.sr.ht/~rjarry/aerc/lib/log"
+	"git.sr.ht/~rockorager/vaxis"
 	"github.com/go-ini/ini"
 )
 
@@ -41,14 +43,15 @@ type BindingConfigContext struct {
 }
 
 type KeyStroke struct {
-	Modifiers tcell.ModMask
-	Key       tcell.Key
-	Rune      rune
+	Modifiers vaxis.ModifierMask
+	Key       rune
 }
 
 type Binding struct {
 	Output []KeyStroke
 	Input  []KeyStroke
+
+	Annotation string
 }
 
 type KeyBindings struct {
@@ -82,9 +85,9 @@ type BindingSearchResult int
 func defaultBindsConfig() *BindingConfig {
 	// These bindings are not configurable
 	wizard := NewKeyBindings()
-	wizard.ExKey = KeyStroke{Key: tcell.KeyCtrlE}
+	wizard.ExKey = KeyStroke{Key: 'e', Modifiers: vaxis.ModCtrl}
 	wizard.Globals = false
-	quit, _ := ParseBinding("<C-q>", ":quit<Enter>")
+	quit, _ := ParseBinding("<C-q>", ":quit<Enter>", "Quit aerc")
 	wizard.Add(quit)
 	return &BindingConfig{
 		Global:                 NewKeyBindings(),
@@ -105,6 +108,10 @@ func parseBindsFromFile(root string, filename string) error {
 	log.Debugf("Parsing key bindings configuration from %s", filename)
 	binds, err := ini.LoadSources(ini.LoadOptions{
 		KeyValueDelimiters: "=",
+		// IgnoreInlineComment is set to true which tells ini's parser
+		// to treat comments (#) on the same line as part of the value;
+		// hence we need cut the comment off ourselves later
+		IgnoreInlineComment: true,
 	}, filename)
 	if err != nil {
 		return err
@@ -167,6 +174,9 @@ func parseBinds(root string, filename string) error {
 func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 	bindings := NewKeyBindings()
 	for key, value := range sec.KeysHash() {
+		var annotation string
+		value, annotation, _ = strings.Cut(value, " # ")
+		value = strings.TrimSpace(value)
 		switch key {
 		case "$ex":
 			strokes, err := ParseKeyStrokes(value)
@@ -195,7 +205,8 @@ func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 			}
 			bindings.CompleteKey = strokes[0]
 		default:
-			binding, err := ParseBinding(key, value)
+			annotation = strings.TrimSpace(annotation)
+			binding, err := ParseBinding(key, value, annotation)
 			if err != nil {
 				return nil, err
 			}
@@ -286,8 +297,8 @@ func LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings) error
 
 func NewKeyBindings() *KeyBindings {
 	return &KeyBindings{
-		ExKey:            KeyStroke{tcell.ModNone, tcell.KeyRune, ':'},
-		CompleteKey:      KeyStroke{tcell.ModNone, tcell.KeyTab, 0},
+		ExKey:            KeyStroke{0, ':'},
+		CompleteKey:      KeyStroke{0, vaxis.KeyTab},
 		Globals:          true,
 		contextualCache:  make(map[bindsContextKey]*KeyBindings),
 		contextualCounts: make(map[bindsContextType]int),
@@ -353,6 +364,12 @@ func MergeBindings(bindings ...*KeyBindings) *KeyBindings {
 	merged.ExKey = bindings[0].ExKey
 	merged.CompleteKey = bindings[0].CompleteKey
 	merged.Globals = bindings[0].Globals
+	for _, b := range bindings {
+		merged.contextualBinds = append(merged.contextualBinds, b.contextualBinds...)
+		for t, c := range b.contextualCounts {
+			merged.contextualCounts[t] += c
+		}
+	}
 	return merged
 }
 
@@ -415,11 +432,6 @@ func (bindings *KeyBindings) GetBinding(
 			if stroke.Key != binding.Input[i].Key {
 				goto next
 			}
-			if stroke.Key == tcell.KeyRune &&
-				stroke.Rune != binding.Input[i].Rune {
-
-				goto next
-			}
 		}
 		if len(binding.Input) != len(input) {
 			incomplete = true
@@ -448,9 +460,6 @@ func (bindings *KeyBindings) GetReverseBindings(output []KeyStroke) [][]KeyStrok
 			if stroke.Key != binding.Output[i].Key {
 				goto next
 			}
-			if stroke.Key == tcell.KeyRune && stroke.Rune != binding.Output[i].Rune {
-				goto next
-			}
 		}
 		inputs = append(inputs, binding.Input)
 	next:
@@ -464,12 +473,10 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 	for _, stroke := range keystrokes {
 		s := ""
 		for name, ks := range keyNames {
-			if ks.Modifiers == stroke.Modifiers && ks.Key == stroke.Key && ks.Rune == stroke.Rune {
+			if (ks.Modifiers == stroke.Modifiers || ks.Modifiers == vaxis.ModifierMask(0)) && ks.Key == stroke.Key {
 				switch name {
-				case "cr", "c-m":
+				case "cr":
 					s = "<enter>"
-				case "c-i":
-					s = "<tab>"
 				case "space":
 					s = " "
 				case "semicolon":
@@ -477,11 +484,23 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 				default:
 					s = fmt.Sprintf("<%s>", name)
 				}
+				// remove any modifiers this named key comes
+				// with so we format properly
+				stroke.Modifiers &^= ks.Modifiers
 				break
 			}
 		}
-		if s == "" && stroke.Key == tcell.KeyRune {
-			s = string(stroke.Rune)
+		if stroke.Modifiers&vaxis.ModCtrl > 0 {
+			sb.WriteString("c-")
+		}
+		if stroke.Modifiers&vaxis.ModAlt > 0 {
+			sb.WriteString("a-")
+		}
+		if stroke.Modifiers&vaxis.ModShift > 0 {
+			sb.WriteString("s-")
+		}
+		if s == "" && stroke.Key < unicode.MaxRune {
+			s = string(stroke.Key)
 		}
 		sb.WriteString(s)
 	}
@@ -501,219 +520,128 @@ func FormatKeyStrokes(keystrokes []KeyStroke) string {
 var spaceTrimRe = regexp.MustCompile(`^(\s*)(.*?)(\s*)$`)
 
 var keyNames = map[string]KeyStroke{
-	"space":     {tcell.ModNone, tcell.KeyRune, ' '},
-	"semicolon": {tcell.ModNone, tcell.KeyRune, ';'},
-	"enter":     {tcell.ModNone, tcell.KeyEnter, 0},
-	"c-enter":   {tcell.ModCtrl, tcell.KeyEnter, 0},
-	"a-enter":   {tcell.ModAlt, tcell.KeyEnter, 0},
-	"up":        {tcell.ModNone, tcell.KeyUp, 0},
-	"c-up":      {tcell.ModCtrl, tcell.KeyUp, 0},
-	"a-up":      {tcell.ModAlt, tcell.KeyUp, 0},
-	"down":      {tcell.ModNone, tcell.KeyDown, 0},
-	"c-down":    {tcell.ModCtrl, tcell.KeyDown, 0},
-	"a-down":    {tcell.ModAlt, tcell.KeyDown, 0},
-	"right":     {tcell.ModNone, tcell.KeyRight, 0},
-	"c-right":   {tcell.ModCtrl, tcell.KeyRight, 0},
-	"a-right":   {tcell.ModAlt, tcell.KeyRight, 0},
-	"left":      {tcell.ModNone, tcell.KeyLeft, 0},
-	"c-left":    {tcell.ModCtrl, tcell.KeyLeft, 0},
-	"a-left":    {tcell.ModAlt, tcell.KeyLeft, 0},
-	"upleft":    {tcell.ModNone, tcell.KeyUpLeft, 0},
-	"upright":   {tcell.ModNone, tcell.KeyUpRight, 0},
-	"downleft":  {tcell.ModNone, tcell.KeyDownLeft, 0},
-	"downright": {tcell.ModNone, tcell.KeyDownRight, 0},
-	"center":    {tcell.ModNone, tcell.KeyCenter, 0},
-	"pgup":      {tcell.ModNone, tcell.KeyPgUp, 0},
-	"c-pgup":    {tcell.ModCtrl, tcell.KeyPgUp, 0},
-	"a-pgup":    {tcell.ModAlt, tcell.KeyPgUp, 0},
-	"pgdn":      {tcell.ModNone, tcell.KeyPgDn, 0},
-	"c-pgdn":    {tcell.ModCtrl, tcell.KeyPgDn, 0},
-	"a-pgdn":    {tcell.ModAlt, tcell.KeyPgDn, 0},
-	"home":      {tcell.ModNone, tcell.KeyHome, 0},
-	"end":       {tcell.ModNone, tcell.KeyEnd, 0},
-	"insert":    {tcell.ModNone, tcell.KeyInsert, 0},
-	"delete":    {tcell.ModNone, tcell.KeyDelete, 0},
-	"c-delete":  {tcell.ModCtrl, tcell.KeyDelete, 0},
-	"a-delete":  {tcell.ModAlt, tcell.KeyDelete, 0},
-	"backspace": {tcell.ModNone, tcell.KeyBackspace2, 0},
-	"help":      {tcell.ModNone, tcell.KeyHelp, 0},
-	"exit":      {tcell.ModNone, tcell.KeyExit, 0},
-	"clear":     {tcell.ModNone, tcell.KeyClear, 0},
-	"cancel":    {tcell.ModNone, tcell.KeyCancel, 0},
-	"print":     {tcell.ModNone, tcell.KeyPrint, 0},
-	"pause":     {tcell.ModNone, tcell.KeyPause, 0},
-	"backtab":   {tcell.ModNone, tcell.KeyBacktab, 0},
-	"f1":        {tcell.ModNone, tcell.KeyF1, 0},
-	"f2":        {tcell.ModNone, tcell.KeyF2, 0},
-	"f3":        {tcell.ModNone, tcell.KeyF3, 0},
-	"f4":        {tcell.ModNone, tcell.KeyF4, 0},
-	"f5":        {tcell.ModNone, tcell.KeyF5, 0},
-	"f6":        {tcell.ModNone, tcell.KeyF6, 0},
-	"f7":        {tcell.ModNone, tcell.KeyF7, 0},
-	"f8":        {tcell.ModNone, tcell.KeyF8, 0},
-	"f9":        {tcell.ModNone, tcell.KeyF9, 0},
-	"f10":       {tcell.ModNone, tcell.KeyF10, 0},
-	"f11":       {tcell.ModNone, tcell.KeyF11, 0},
-	"f12":       {tcell.ModNone, tcell.KeyF12, 0},
-	"f13":       {tcell.ModNone, tcell.KeyF13, 0},
-	"f14":       {tcell.ModNone, tcell.KeyF14, 0},
-	"f15":       {tcell.ModNone, tcell.KeyF15, 0},
-	"f16":       {tcell.ModNone, tcell.KeyF16, 0},
-	"f17":       {tcell.ModNone, tcell.KeyF17, 0},
-	"f18":       {tcell.ModNone, tcell.KeyF18, 0},
-	"f19":       {tcell.ModNone, tcell.KeyF19, 0},
-	"f20":       {tcell.ModNone, tcell.KeyF20, 0},
-	"f21":       {tcell.ModNone, tcell.KeyF21, 0},
-	"f22":       {tcell.ModNone, tcell.KeyF22, 0},
-	"f23":       {tcell.ModNone, tcell.KeyF23, 0},
-	"f24":       {tcell.ModNone, tcell.KeyF24, 0},
-	"f25":       {tcell.ModNone, tcell.KeyF25, 0},
-	"f26":       {tcell.ModNone, tcell.KeyF26, 0},
-	"f27":       {tcell.ModNone, tcell.KeyF27, 0},
-	"f28":       {tcell.ModNone, tcell.KeyF28, 0},
-	"f29":       {tcell.ModNone, tcell.KeyF29, 0},
-	"f30":       {tcell.ModNone, tcell.KeyF30, 0},
-	"f31":       {tcell.ModNone, tcell.KeyF31, 0},
-	"f32":       {tcell.ModNone, tcell.KeyF32, 0},
-	"f33":       {tcell.ModNone, tcell.KeyF33, 0},
-	"f34":       {tcell.ModNone, tcell.KeyF34, 0},
-	"f35":       {tcell.ModNone, tcell.KeyF35, 0},
-	"f36":       {tcell.ModNone, tcell.KeyF36, 0},
-	"f37":       {tcell.ModNone, tcell.KeyF37, 0},
-	"f38":       {tcell.ModNone, tcell.KeyF38, 0},
-	"f39":       {tcell.ModNone, tcell.KeyF39, 0},
-	"f40":       {tcell.ModNone, tcell.KeyF40, 0},
-	"f41":       {tcell.ModNone, tcell.KeyF41, 0},
-	"f42":       {tcell.ModNone, tcell.KeyF42, 0},
-	"f43":       {tcell.ModNone, tcell.KeyF43, 0},
-	"f44":       {tcell.ModNone, tcell.KeyF44, 0},
-	"f45":       {tcell.ModNone, tcell.KeyF45, 0},
-	"f46":       {tcell.ModNone, tcell.KeyF46, 0},
-	"f47":       {tcell.ModNone, tcell.KeyF47, 0},
-	"f48":       {tcell.ModNone, tcell.KeyF48, 0},
-	"f49":       {tcell.ModNone, tcell.KeyF49, 0},
-	"f50":       {tcell.ModNone, tcell.KeyF50, 0},
-	"f51":       {tcell.ModNone, tcell.KeyF51, 0},
-	"f52":       {tcell.ModNone, tcell.KeyF52, 0},
-	"f53":       {tcell.ModNone, tcell.KeyF53, 0},
-	"f54":       {tcell.ModNone, tcell.KeyF54, 0},
-	"f55":       {tcell.ModNone, tcell.KeyF55, 0},
-	"f56":       {tcell.ModNone, tcell.KeyF56, 0},
-	"f57":       {tcell.ModNone, tcell.KeyF57, 0},
-	"f58":       {tcell.ModNone, tcell.KeyF58, 0},
-	"f59":       {tcell.ModNone, tcell.KeyF59, 0},
-	"f60":       {tcell.ModNone, tcell.KeyF60, 0},
-	"f61":       {tcell.ModNone, tcell.KeyF61, 0},
-	"f62":       {tcell.ModNone, tcell.KeyF62, 0},
-	"f63":       {tcell.ModNone, tcell.KeyF63, 0},
-	"f64":       {tcell.ModNone, tcell.KeyF64, 0},
-	"c-space":   {tcell.ModCtrl, tcell.KeyCtrlSpace, 0},
-	"c-a":       {tcell.ModCtrl, tcell.KeyCtrlA, 0},
-	"c-b":       {tcell.ModCtrl, tcell.KeyCtrlB, 0},
-	"c-c":       {tcell.ModCtrl, tcell.KeyCtrlC, 0},
-	"c-d":       {tcell.ModCtrl, tcell.KeyCtrlD, 0},
-	"c-e":       {tcell.ModCtrl, tcell.KeyCtrlE, 0},
-	"c-f":       {tcell.ModCtrl, tcell.KeyCtrlF, 0},
-	"c-g":       {tcell.ModCtrl, tcell.KeyCtrlG, 0},
-	"c-h":       {tcell.ModNone, tcell.KeyCtrlH, 0},
-	"c-i":       {tcell.ModNone, tcell.KeyCtrlI, 0},
-	"c-j":       {tcell.ModCtrl, tcell.KeyCtrlJ, 0},
-	"c-k":       {tcell.ModCtrl, tcell.KeyCtrlK, 0},
-	"c-l":       {tcell.ModCtrl, tcell.KeyCtrlL, 0},
-	"c-m":       {tcell.ModNone, tcell.KeyCtrlM, 0},
-	"c-n":       {tcell.ModCtrl, tcell.KeyCtrlN, 0},
-	"c-o":       {tcell.ModCtrl, tcell.KeyCtrlO, 0},
-	"c-p":       {tcell.ModCtrl, tcell.KeyCtrlP, 0},
-	"c-q":       {tcell.ModCtrl, tcell.KeyCtrlQ, 0},
-	"c-r":       {tcell.ModCtrl, tcell.KeyCtrlR, 0},
-	"c-s":       {tcell.ModCtrl, tcell.KeyCtrlS, 0},
-	"c-t":       {tcell.ModCtrl, tcell.KeyCtrlT, 0},
-	"c-u":       {tcell.ModCtrl, tcell.KeyCtrlU, 0},
-	"c-v":       {tcell.ModCtrl, tcell.KeyCtrlV, 0},
-	"c-w":       {tcell.ModCtrl, tcell.KeyCtrlW, 0},
-	"c-x":       {tcell.ModCtrl, tcell.KeyCtrlX, rune(tcell.KeyCAN)},
-	"c-y":       {tcell.ModCtrl, tcell.KeyCtrlY, 0}, // TODO: runes for the rest
-	"c-z":       {tcell.ModCtrl, tcell.KeyCtrlZ, 0},
-	"c-]":       {tcell.ModCtrl, tcell.KeyCtrlRightSq, 0},
-	"c-\\":      {tcell.ModCtrl, tcell.KeyCtrlBackslash, 0},
-	"c-[":       {tcell.ModCtrl, tcell.KeyCtrlLeftSq, 0},
-	"c-^":       {tcell.ModCtrl, tcell.KeyCtrlCarat, 0},
-	"c-_":       {tcell.ModCtrl, tcell.KeyCtrlUnderscore, 0},
-	"a-space":   {tcell.ModAlt, tcell.KeyRune, ' '},
-	"a-0":       {tcell.ModAlt, tcell.KeyRune, '0'},
-	"a-1":       {tcell.ModAlt, tcell.KeyRune, '1'},
-	"a-2":       {tcell.ModAlt, tcell.KeyRune, '2'},
-	"a-3":       {tcell.ModAlt, tcell.KeyRune, '3'},
-	"a-4":       {tcell.ModAlt, tcell.KeyRune, '4'},
-	"a-5":       {tcell.ModAlt, tcell.KeyRune, '5'},
-	"a-6":       {tcell.ModAlt, tcell.KeyRune, '6'},
-	"a-7":       {tcell.ModAlt, tcell.KeyRune, '7'},
-	"a-8":       {tcell.ModAlt, tcell.KeyRune, '8'},
-	"a-9":       {tcell.ModAlt, tcell.KeyRune, '9'},
-	"a-a":       {tcell.ModAlt, tcell.KeyRune, 'a'},
-	"a-b":       {tcell.ModAlt, tcell.KeyRune, 'b'},
-	"a-c":       {tcell.ModAlt, tcell.KeyRune, 'c'},
-	"a-d":       {tcell.ModAlt, tcell.KeyRune, 'd'},
-	"a-e":       {tcell.ModAlt, tcell.KeyRune, 'e'},
-	"a-f":       {tcell.ModAlt, tcell.KeyRune, 'f'},
-	"a-g":       {tcell.ModAlt, tcell.KeyRune, 'g'},
-	"a-h":       {tcell.ModAlt, tcell.KeyRune, 'h'},
-	"a-i":       {tcell.ModAlt, tcell.KeyRune, 'i'},
-	"a-j":       {tcell.ModAlt, tcell.KeyRune, 'j'},
-	"a-k":       {tcell.ModAlt, tcell.KeyRune, 'k'},
-	"a-l":       {tcell.ModAlt, tcell.KeyRune, 'l'},
-	"a-m":       {tcell.ModAlt, tcell.KeyRune, 'm'},
-	"a-n":       {tcell.ModAlt, tcell.KeyRune, 'n'},
-	"a-o":       {tcell.ModAlt, tcell.KeyRune, 'o'},
-	"a-p":       {tcell.ModAlt, tcell.KeyRune, 'p'},
-	"a-q":       {tcell.ModAlt, tcell.KeyRune, 'q'},
-	"a-r":       {tcell.ModAlt, tcell.KeyRune, 'r'},
-	"a-s":       {tcell.ModAlt, tcell.KeyRune, 's'},
-	"a-t":       {tcell.ModAlt, tcell.KeyRune, 't'},
-	"a-u":       {tcell.ModAlt, tcell.KeyRune, 'u'},
-	"a-v":       {tcell.ModAlt, tcell.KeyRune, 'v'},
-	"a-w":       {tcell.ModAlt, tcell.KeyRune, 'w'},
-	"a-x":       {tcell.ModAlt, tcell.KeyRune, 'x'},
-	"a-y":       {tcell.ModAlt, tcell.KeyRune, 'y'},
-	"a-z":       {tcell.ModAlt, tcell.KeyRune, 'z'},
-	"a-]":       {tcell.ModAlt, tcell.KeyRune, ']'},
-	"a-\\":      {tcell.ModAlt, tcell.KeyRune, '\\'},
-	"a-[":       {tcell.ModAlt, tcell.KeyRune, '['},
-	"a-^":       {tcell.ModAlt, tcell.KeyRune, '^'},
-	"a-_":       {tcell.ModAlt, tcell.KeyRune, '_'},
-	"nul":       {tcell.ModNone, tcell.KeyNUL, 0},
-	"soh":       {tcell.ModNone, tcell.KeySOH, 0},
-	"stx":       {tcell.ModNone, tcell.KeySTX, 0},
-	"etx":       {tcell.ModNone, tcell.KeyETX, 0},
-	"eot":       {tcell.ModNone, tcell.KeyEOT, 0},
-	"enq":       {tcell.ModNone, tcell.KeyENQ, 0},
-	"ack":       {tcell.ModNone, tcell.KeyACK, 0},
-	"bel":       {tcell.ModNone, tcell.KeyBEL, 0},
-	"bs":        {tcell.ModNone, tcell.KeyBS, 0},
-	"tab":       {tcell.ModNone, tcell.KeyTAB, 0},
-	"lf":        {tcell.ModNone, tcell.KeyLF, 0},
-	"vt":        {tcell.ModNone, tcell.KeyVT, 0},
-	"ff":        {tcell.ModNone, tcell.KeyFF, 0},
-	"cr":        {tcell.ModNone, tcell.KeyCR, 0},
-	"so":        {tcell.ModNone, tcell.KeySO, 0},
-	"si":        {tcell.ModNone, tcell.KeySI, 0},
-	"dle":       {tcell.ModNone, tcell.KeyDLE, 0},
-	"dc1":       {tcell.ModNone, tcell.KeyDC1, 0},
-	"dc2":       {tcell.ModNone, tcell.KeyDC2, 0},
-	"dc3":       {tcell.ModNone, tcell.KeyDC3, 0},
-	"dc4":       {tcell.ModNone, tcell.KeyDC4, 0},
-	"nak":       {tcell.ModNone, tcell.KeyNAK, 0},
-	"syn":       {tcell.ModNone, tcell.KeySYN, 0},
-	"etb":       {tcell.ModNone, tcell.KeyETB, 0},
-	"can":       {tcell.ModNone, tcell.KeyCAN, 0},
-	"em":        {tcell.ModNone, tcell.KeyEM, 0},
-	"sub":       {tcell.ModNone, tcell.KeySUB, 0},
-	"esc":       {tcell.ModNone, tcell.KeyESC, 0},
-	"fs":        {tcell.ModNone, tcell.KeyFS, 0},
-	"gs":        {tcell.ModNone, tcell.KeyGS, 0},
-	"rs":        {tcell.ModNone, tcell.KeyRS, 0},
-	"us":        {tcell.ModNone, tcell.KeyUS, 0},
-	"del":       {tcell.ModNone, tcell.KeyDEL, 0},
+	"space":     {vaxis.ModifierMask(0), ' '},
+	"semicolon": {vaxis.ModifierMask(0), ';'},
+	"enter":     {vaxis.ModifierMask(0), vaxis.KeyEnter},
+	"up":        {vaxis.ModifierMask(0), vaxis.KeyUp},
+	"down":      {vaxis.ModifierMask(0), vaxis.KeyDown},
+	"right":     {vaxis.ModifierMask(0), vaxis.KeyRight},
+	"left":      {vaxis.ModifierMask(0), vaxis.KeyLeft},
+	"upleft":    {vaxis.ModifierMask(0), vaxis.KeyUpLeft},
+	"upright":   {vaxis.ModifierMask(0), vaxis.KeyUpRight},
+	"downleft":  {vaxis.ModifierMask(0), vaxis.KeyDownLeft},
+	"downright": {vaxis.ModifierMask(0), vaxis.KeyDownRight},
+	"center":    {vaxis.ModifierMask(0), vaxis.KeyCenter},
+	"pgup":      {vaxis.ModifierMask(0), vaxis.KeyPgUp},
+	"pgdn":      {vaxis.ModifierMask(0), vaxis.KeyPgDown},
+	"home":      {vaxis.ModifierMask(0), vaxis.KeyHome},
+	"end":       {vaxis.ModifierMask(0), vaxis.KeyEnd},
+	"insert":    {vaxis.ModifierMask(0), vaxis.KeyInsert},
+	"delete":    {vaxis.ModifierMask(0), vaxis.KeyDelete},
+	"backspace": {vaxis.ModifierMask(0), vaxis.KeyBackspace},
+	// "help":      {vaxis.ModifierMask(0), vaxis.KeyHelp},
+	"exit":    {vaxis.ModifierMask(0), vaxis.KeyExit},
+	"clear":   {vaxis.ModifierMask(0), vaxis.KeyClear},
+	"cancel":  {vaxis.ModifierMask(0), vaxis.KeyCancel},
+	"print":   {vaxis.ModifierMask(0), vaxis.KeyPrint},
+	"pause":   {vaxis.ModifierMask(0), vaxis.KeyPause},
+	"backtab": {vaxis.ModShift, vaxis.KeyTab},
+	"f1":      {vaxis.ModifierMask(0), vaxis.KeyF01},
+	"f2":      {vaxis.ModifierMask(0), vaxis.KeyF02},
+	"f3":      {vaxis.ModifierMask(0), vaxis.KeyF03},
+	"f4":      {vaxis.ModifierMask(0), vaxis.KeyF04},
+	"f5":      {vaxis.ModifierMask(0), vaxis.KeyF05},
+	"f6":      {vaxis.ModifierMask(0), vaxis.KeyF06},
+	"f7":      {vaxis.ModifierMask(0), vaxis.KeyF07},
+	"f8":      {vaxis.ModifierMask(0), vaxis.KeyF08},
+	"f9":      {vaxis.ModifierMask(0), vaxis.KeyF09},
+	"f10":     {vaxis.ModifierMask(0), vaxis.KeyF10},
+	"f11":     {vaxis.ModifierMask(0), vaxis.KeyF11},
+	"f12":     {vaxis.ModifierMask(0), vaxis.KeyF12},
+	"f13":     {vaxis.ModifierMask(0), vaxis.KeyF13},
+	"f14":     {vaxis.ModifierMask(0), vaxis.KeyF14},
+	"f15":     {vaxis.ModifierMask(0), vaxis.KeyF15},
+	"f16":     {vaxis.ModifierMask(0), vaxis.KeyF16},
+	"f17":     {vaxis.ModifierMask(0), vaxis.KeyF17},
+	"f18":     {vaxis.ModifierMask(0), vaxis.KeyF18},
+	"f19":     {vaxis.ModifierMask(0), vaxis.KeyF19},
+	"f20":     {vaxis.ModifierMask(0), vaxis.KeyF20},
+	"f21":     {vaxis.ModifierMask(0), vaxis.KeyF21},
+	"f22":     {vaxis.ModifierMask(0), vaxis.KeyF22},
+	"f23":     {vaxis.ModifierMask(0), vaxis.KeyF23},
+	"f24":     {vaxis.ModifierMask(0), vaxis.KeyF24},
+	"f25":     {vaxis.ModifierMask(0), vaxis.KeyF25},
+	"f26":     {vaxis.ModifierMask(0), vaxis.KeyF26},
+	"f27":     {vaxis.ModifierMask(0), vaxis.KeyF27},
+	"f28":     {vaxis.ModifierMask(0), vaxis.KeyF28},
+	"f29":     {vaxis.ModifierMask(0), vaxis.KeyF29},
+	"f30":     {vaxis.ModifierMask(0), vaxis.KeyF30},
+	"f31":     {vaxis.ModifierMask(0), vaxis.KeyF31},
+	"f32":     {vaxis.ModifierMask(0), vaxis.KeyF32},
+	"f33":     {vaxis.ModifierMask(0), vaxis.KeyF33},
+	"f34":     {vaxis.ModifierMask(0), vaxis.KeyF34},
+	"f35":     {vaxis.ModifierMask(0), vaxis.KeyF35},
+	"f36":     {vaxis.ModifierMask(0), vaxis.KeyF36},
+	"f37":     {vaxis.ModifierMask(0), vaxis.KeyF37},
+	"f38":     {vaxis.ModifierMask(0), vaxis.KeyF38},
+	"f39":     {vaxis.ModifierMask(0), vaxis.KeyF39},
+	"f40":     {vaxis.ModifierMask(0), vaxis.KeyF40},
+	"f41":     {vaxis.ModifierMask(0), vaxis.KeyF41},
+	"f42":     {vaxis.ModifierMask(0), vaxis.KeyF42},
+	"f43":     {vaxis.ModifierMask(0), vaxis.KeyF43},
+	"f44":     {vaxis.ModifierMask(0), vaxis.KeyF44},
+	"f45":     {vaxis.ModifierMask(0), vaxis.KeyF45},
+	"f46":     {vaxis.ModifierMask(0), vaxis.KeyF46},
+	"f47":     {vaxis.ModifierMask(0), vaxis.KeyF47},
+	"f48":     {vaxis.ModifierMask(0), vaxis.KeyF48},
+	"f49":     {vaxis.ModifierMask(0), vaxis.KeyF49},
+	"f50":     {vaxis.ModifierMask(0), vaxis.KeyF50},
+	"f51":     {vaxis.ModifierMask(0), vaxis.KeyF51},
+	"f52":     {vaxis.ModifierMask(0), vaxis.KeyF52},
+	"f53":     {vaxis.ModifierMask(0), vaxis.KeyF53},
+	"f54":     {vaxis.ModifierMask(0), vaxis.KeyF54},
+	"f55":     {vaxis.ModifierMask(0), vaxis.KeyF55},
+	"f56":     {vaxis.ModifierMask(0), vaxis.KeyF56},
+	"f57":     {vaxis.ModifierMask(0), vaxis.KeyF57},
+	"f58":     {vaxis.ModifierMask(0), vaxis.KeyF58},
+	"f59":     {vaxis.ModifierMask(0), vaxis.KeyF59},
+	"f60":     {vaxis.ModifierMask(0), vaxis.KeyF60},
+	"f61":     {vaxis.ModifierMask(0), vaxis.KeyF61},
+	"f62":     {vaxis.ModifierMask(0), vaxis.KeyF62},
+	"f63":     {vaxis.ModifierMask(0), vaxis.KeyF63},
+	"nul":     {vaxis.ModCtrl, ' '},
+	"soh":     {vaxis.ModCtrl, 'a'},
+	"stx":     {vaxis.ModCtrl, 'b'},
+	"etx":     {vaxis.ModCtrl, 'c'},
+	"eot":     {vaxis.ModCtrl, 'd'},
+	"enq":     {vaxis.ModCtrl, 'e'},
+	"ack":     {vaxis.ModCtrl, 'f'},
+	"bel":     {vaxis.ModCtrl, 'g'},
+	"bs":      {vaxis.ModCtrl, 'h'},
+	"tab":     {vaxis.ModifierMask(0), vaxis.KeyTab},
+	"lf":      {vaxis.ModCtrl, 'j'},
+	"vt":      {vaxis.ModCtrl, 'k'},
+	"ff":      {vaxis.ModCtrl, 'l'},
+	"cr":      {vaxis.ModifierMask(0), vaxis.KeyEnter},
+	"so":      {vaxis.ModCtrl, 'n'},
+	"si":      {vaxis.ModCtrl, 'o'},
+	"dle":     {vaxis.ModCtrl, 'p'},
+	"dc1":     {vaxis.ModCtrl, 'q'},
+	"dc2":     {vaxis.ModCtrl, 'r'},
+	"dc3":     {vaxis.ModCtrl, 's'},
+	"dc4":     {vaxis.ModCtrl, 't'},
+	"nak":     {vaxis.ModCtrl, 'u'},
+	"syn":     {vaxis.ModCtrl, 'v'},
+	"etb":     {vaxis.ModCtrl, 'w'},
+	"can":     {vaxis.ModCtrl, 'x'},
+	"em":      {vaxis.ModCtrl, 'y'},
+	"sub":     {vaxis.ModCtrl, 'z'},
+	"esc":     {vaxis.ModifierMask(0), vaxis.KeyEsc},
+	"fs":      {vaxis.ModCtrl, '\\'},
+	"gs":      {vaxis.ModCtrl, ']'},
+	"rs":      {vaxis.ModCtrl, '^'},
+	"us":      {vaxis.ModCtrl, '_'},
+	"del":     {vaxis.ModifierMask(0), vaxis.KeyDelete},
 }
 
 func ParseKeyStrokes(keystrokes string) ([]KeyStroke, error) {
@@ -740,10 +668,36 @@ func ParseKeyStrokes(keystrokes string) ([]KeyStroke, error) {
 				return nil, errors.New("Expected a key name")
 			}
 			name = name[:len(name)-1]
-			if key, ok := keyNames[strings.ToLower(name)]; ok {
-				strokes = append(strokes, key)
-			} else {
-				return nil, fmt.Errorf("Unknown key '%s'", name)
+			args := strings.Split(name, "-")
+			// check if the last char was a '-' and we'll add it
+			// back. We check for "--" in case it was an invalid
+			// keystroke (ie <C->)
+			if strings.HasSuffix(name, "--") {
+				args = append(args, "-")
+			}
+			ks := KeyStroke{}
+			for i, arg := range args {
+				if i == len(args)-1 {
+					key, ok := keyNames[strings.ToLower(arg)]
+					if !ok {
+						r, n := utf8.DecodeRuneInString(arg)
+						if n != len(arg) {
+							return nil, fmt.Errorf("Unknown key '%s'", name)
+						}
+						key = KeyStroke{Key: r}
+					}
+					ks.Key = key.Key
+					ks.Modifiers |= key.Modifiers
+					strokes = append(strokes, ks)
+				}
+				switch strings.ToLower(arg) {
+				case "s", "S":
+					ks.Modifiers |= vaxis.ModShift
+				case "a", "A":
+					ks.Modifiers |= vaxis.ModAlt
+				case "c", "C":
+					ks.Modifiers |= vaxis.ModCtrl
+				}
 			}
 		case '>':
 			return nil, errors.New("Found '>' without '<'")
@@ -757,16 +711,15 @@ func ParseKeyStrokes(keystrokes string) ([]KeyStroke, error) {
 			fallthrough
 		default:
 			strokes = append(strokes, KeyStroke{
-				Modifiers: tcell.ModNone,
-				Key:       tcell.KeyRune,
-				Rune:      tok,
+				Modifiers: vaxis.ModifierMask(0),
+				Key:       tok,
 			})
 		}
 	}
 	return strokes, nil
 }
 
-func ParseBinding(input, output string) (*Binding, error) {
+func ParseBinding(input, output, annotation string) (*Binding, error) {
 	in, err := ParseKeyStrokes(input)
 	if err != nil {
 		return nil, err
@@ -776,7 +729,8 @@ func ParseBinding(input, output string) (*Binding, error) {
 		return nil, err
 	}
 	return &Binding{
-		Input:  in,
-		Output: out,
+		Input:      in,
+		Output:     out,
+		Annotation: annotation,
 	}, nil
 }
