@@ -3,9 +3,10 @@ package config
 import (
 	"fmt"
 	"os"
+	"path/filepath"
 
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/xdg"
-	"git.sr.ht/~rjarry/aerc/log"
 	"github.com/go-ini/ini"
 	"github.com/mattn/go-isatty"
 )
@@ -17,6 +18,8 @@ type GeneralConfig struct {
 	LogFile            string       `ini:"log-file"`
 	LogLevel           log.LogLevel `ini:"log-level" default:"info" parse:"ParseLogLevel"`
 	DisableIPC         bool         `ini:"disable-ipc"`
+	DisableIPCMailto   bool         `ini:"disable-ipc-mailto"`
+	DisableIPCMbox     bool         `ini:"disable-ipc-mbox"`
 	EnableOSC8         bool         `ini:"enable-osc8" default:"false"`
 	Term               string       `ini:"term" default:"xterm-256color"`
 	DefaultMenuCmd     string       `ini:"default-menu-cmd"`
@@ -37,6 +40,10 @@ func parseGeneral(file *ini.File) error {
 	} else if General.LogFile != "" {
 		var err error
 		path := xdg.ExpandHome(General.LogFile)
+		err = os.MkdirAll(filepath.Dir(path), 0o700)
+		if err != nil {
+			return fmt.Errorf("log-file: %w", err)
+		}
 		logFile, err = os.OpenFile(path,
 			os.O_CREATE|os.O_APPEND|os.O_WRONLY, 0o600)
 		if err != nil {
