@@ -4,7 +4,7 @@ import (
 	"os"
 	"os/exec"
 
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 )
 
 func RunHook(h HookType) error {
