@@ -10,8 +10,8 @@ import (
 	_ "github.com/emersion/go-message/charset"
 
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/rfc822"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -73,7 +73,8 @@ func NewMessageStoreView(messageInfo *models.MessageInfo, setSeen bool,
 	innerCb func(MessageView, error),
 ) {
 	cb := func(msv MessageView, err error) {
-		if msv != nil && setSeen && err == nil {
+		if msv != nil && setSeen && err == nil &&
+			!messageInfo.Flags.Has(models.SeenFlag) {
 			store.Flag([]uint32{messageInfo.Uid}, models.SeenFlag, true, nil)
 		}
 		innerCb(msv, err)
