@@ -6,13 +6,15 @@ import (
 	"sync/atomic"
 	"syscall"
 
-	"github.com/gdamore/tcell/v2"
+	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/log"
+	"git.sr.ht/~rockorager/vaxis"
 )
 
 // Use unbuffered channels (always blocking unless somebody can read
-// immediately) We are merely using this as a proxy to tcell screen internal
-// event channel.
-var Events = make(chan tcell.Event)
+// immediately) We are merely using this as a proxy to the internal vaxis event
+// channel.
+var Events = make(chan vaxis.Event)
 
 var Quit = make(chan struct{})
 
@@ -38,7 +40,7 @@ func Invalidate() {
 var state struct {
 	content DrawableInteractive
 	ctx     *Context
-	screen  tcell.Screen
+	vx      *vaxis.Vaxis
 	popover *Popover
 	dirty   uint32 // == 1 if render has been queued in Redraw channel
 	// == 1 if suspend is pending
@@ -46,32 +48,36 @@ var state struct {
 }
 
 func Initialize(content DrawableInteractive) error {
-	screen, err := tcell.NewScreen()
+	opts := vaxis.Options{
+		DisableMouse: !config.Ui.MouseEnabled,
+		CSIuBitMask:  vaxis.CSIuDisambiguate,
+		WithTTY:      "/dev/tty",
+	}
+	vx, err := vaxis.New(opts)
 	if err != nil {
 		return err
 	}
 
-	if err = screen.Init(); err != nil {
-		return err
-	}
-
-	screen.Clear()
-	screen.HideCursor()
-	screen.EnablePaste()
-
-	width, height := screen.Size()
+	vx.Window().Clear()
+	vx.HideCursor()
 
 	state.content = content
-	state.screen = screen
-	state.ctx = NewContext(width, height, state.screen, onPopover)
+	state.vx = vx
+	state.ctx = NewContext(state.vx, onPopover)
+	vx.SetTitle("aerc")
 
 	Invalidate()
 	if beeper, ok := content.(DrawableInteractiveBeeper); ok {
-		beeper.OnBeep(screen.Beep)
+		beeper.OnBeep(vx.Bell)
 	}
 	content.Focus(true)
 
-	go state.screen.ChannelEvents(Events, Quit)
+	go func() {
+		defer log.PanicHandler()
+		for event := range vx.Events() {
+			Events <- event
+		}
+	}()
 
 	return nil
 }
@@ -95,7 +101,7 @@ func QueueSuspend() {
 func Suspend() error {
 	var err error
 	if atomic.SwapUint32(&state.suspending, 0) != 0 {
-		err = state.screen.Suspend()
+		err = state.vx.Suspend()
 		if err == nil {
 			sigcont := make(chan os.Signal, 1)
 			signal.Notify(sigcont, syscall.SIGCONT)
@@ -104,45 +110,52 @@ func Suspend() error {
 				<-sigcont
 			}
 			signal.Reset(syscall.SIGCONT)
-			err = state.screen.Resume()
+			err = state.vx.Resume()
 			state.content.Draw(state.ctx)
-			state.screen.Show()
+			state.vx.Render()
 		}
 	}
 	return err
 }
 
 func Close() {
-	state.screen.Fini()
+	state.vx.Close()
 }
 
 func Render() {
 	if atomic.SwapUint32(&state.dirty, 0) != 0 {
+		state.vx.Window().Clear()
 		// reset popover for the next Draw
 		state.popover = nil
+		state.vx.HideCursor()
 		state.content.Draw(state.ctx)
 		if state.popover != nil {
 			// if the Draw resulted in a popover, draw it
 			state.popover.Draw(state.ctx)
 		}
-		state.screen.Show()
+		state.vx.Render()
 	}
 }
 
-func EnableMouse() {
-	state.screen.EnableMouse()
-}
-
-func HandleEvent(event tcell.Event) {
-	if event, ok := event.(*tcell.EventResize); ok {
-		state.screen.Clear()
-		width, height := event.Size()
-		state.ctx = NewContext(width, height, state.screen, onPopover)
+func HandleEvent(event vaxis.Event) {
+	switch event := event.(type) {
+	case vaxis.Resize:
+		state.ctx = NewContext(state.vx, onPopover)
 		Invalidate()
-	}
-	// if we have a popover, and it can handle the event, it does so
-	if state.popover == nil || !state.popover.Event(event) {
-		// otherwise, we send the event to the main content
-		state.content.Event(event)
+	case vaxis.Redraw:
+		Invalidate()
+	default:
+		// We never care about num or caps lock. Remove them so it
+		// doesn't interefere with key matching
+		if key, ok := event.(vaxis.Key); ok {
+			key.Modifiers &^= vaxis.ModCapsLock
+			key.Modifiers &^= vaxis.ModNumLock
+			event = key
+		}
+		// if we have a popover, and it can handle the event, it does so
+		if state.popover == nil || !state.popover.Event(event) {
+			// otherwise, we send the event to the main content
+			state.content.Event(event)
+		}
 	}
 }
