@@ -1,7 +1,6 @@
 package main
 
 import (
-	"bytes"
 	"context"
 	"errors"
 	"fmt"
@@ -14,8 +13,6 @@ import (
 	"time"
 
 	"git.sr.ht/~rjarry/go-opt"
-	"github.com/mattn/go-isatty"
-	"github.com/xo/terminfo"
 
 	"git.sr.ht/~rjarry/aerc/app"
 	"git.sr.ht/~rjarry/aerc/commands"
@@ -24,9 +21,9 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/hooks"
 	"git.sr.ht/~rjarry/aerc/lib/ipc"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 
@@ -98,34 +95,14 @@ func buildInfo() string {
 	return info
 }
 
-func setWindowTitle() {
-	log.Tracef("Parsing terminfo")
-	ti, err := terminfo.LoadFromEnv()
-	if err != nil {
-		log.Warnf("Cannot get terminfo: %v", err)
-		return
-	}
-
-	if !ti.Has(terminfo.HasStatusLine) {
-		log.Infof("Terminal does not have status line support")
-		return
-	}
-
-	log.Debugf("Setting terminal title")
-	buf := new(bytes.Buffer)
-	ti.Fprintf(buf, terminfo.ToStatusLine)
-	fmt.Fprint(buf, "aerc")
-	ti.Fprintf(buf, terminfo.FromStatusLine)
-	os.Stderr.Write(buf.Bytes())
-}
-
 type Opts struct {
-	Help         bool     `opt:"-h" action:"ShowHelp"`
-	Version      bool     `opt:"-v" action:"ShowVersion"`
-	Accounts     []string `opt:"-a" action:"ParseAccounts" metavar:"<account>"`
-	ConfAerc     string   `opt:"--aerc-conf"`
-	ConfAccounts string   `opt:"--accounts-conf"`
-	ConfBinds    string   `opt:"--binds-conf"`
+	Help         bool     `opt:"-h,--help" action:"ShowHelp"`
+	Version      bool     `opt:"-v,--version" action:"ShowVersion"`
+	Accounts     []string `opt:"-a,--account" action:"ParseAccounts" metavar:"<name>"`
+	ConfAerc     string   `opt:"-C,--aerc-conf" metavar:"<file>"`
+	ConfAccounts string   `opt:"-A,--accounts-conf" metavar:"<file>"`
+	ConfBinds    string   `opt:"-B,--binds-conf" metavar:"<file>"`
+	NoIPC        bool     `opt:"-I,--no-ipc"`
 	Command      []string `opt:"..." required:"false" metavar:"mailto:<address> | mbox:<file> | :<command...>"`
 }
 
@@ -136,15 +113,21 @@ Aerc is an email client for your terminal.
 
 Options:
 
-  -h                 Show this help message and exit.
-  -v                 Print version information.
-  -a <account>       Load only the named account, as opposed to all configured
+  -h, --help         Show this help message and exit.
+  -v, --version      Print version information.
+  -a <name>, --account <name>
+                     Load only the named account, as opposed to all configured
                      accounts. It can also be a comma separated list of names.
                      This option may be specified multiple times. The account
                      order will be preserved.
-  --aerc-conf        Path to configuration file to be used instead of the default.
-  --accounts-conf    Path to configuration file to be used instead of the default.
-  --binds-conf       Path to configuration file to be used instead of the default.
+  -C <file>, --aerc-conf <file>
+                     Path to configuration file to be used instead of the default.
+  -A <file>, --accounts-conf <file>
+                     Path to configuration file to be used instead of the default.
+  -B <file>, --binds-conf <file>
+                     Path to configuration file to be used instead of the default.
+  -I, --no-ipc       Run any commands in this aerc instance, and don't create a
+                     socket for other aerc instances to communicate with this one.
   mailto:<address>   Open the composer with the address(es) in the To field.
                      If aerc is already running, the composer is started in
                      this instance, otherwise aerc will be started.
@@ -182,22 +165,38 @@ func main() {
 	if err != nil {
 		die("%s", err)
 	}
-	retryExec := false
-	if len(opts.Command) > 0 {
-		err := ipc.ConnectAndExec(opts.Command)
-		if err == nil {
-			return // other aerc instance takes over
-		}
-		fmt.Fprintf(os.Stderr, "Failed to communicate to aerc: %v\n", err)
-		// continue with setting up a new aerc instance and retry after init
-		retryExec = true
+	switch {
+	case len(opts.Command) == 0:
+		break
+	case strings.HasPrefix(opts.Command[0], ":"):
+	case strings.HasPrefix(opts.Command[0], "mailto:"):
+	case strings.HasPrefix(opts.Command[0], "mbox:"):
+		break
+	default:
+		die("unknown argument: %s", opts.Command[0])
 	}
 
 	err = config.LoadConfigFromFile(
 		nil, opts.Accounts, opts.ConfAerc, opts.ConfBinds, opts.ConfAccounts,
 	)
 	if err != nil {
-		die("failed to load config: %s", err)
+		die("%s", err)
+	}
+
+	noIPC := opts.NoIPC || config.General.DisableIPC
+
+	if len(opts.Command) > 0 && !noIPC &&
+		!(config.General.DisableIPCMailto && strings.HasPrefix(opts.Command[0], "mailto:")) &&
+		!(config.General.DisableIPCMbox && strings.HasPrefix(opts.Command[0], "mbox:")) {
+
+		response, err := ipc.ConnectAndExec(opts.Command)
+		if err == nil {
+			if response.Error != "" {
+				fmt.Printf("response: %s\n", response.Error)
+			}
+			return // other aerc instance takes over
+		}
+		// continue with setting up a new aerc instance and retry after init
 	}
 
 	log.Infof("Starting up version %s", log.BuildInfo)
@@ -223,36 +222,37 @@ func main() {
 	}
 	close(deferLoop)
 
-	if config.Ui.MouseEnabled {
-		ui.EnableMouse()
-	}
 	startup, startupDone := context.WithCancel(context.Background())
 
-	as, err := ipc.StartServer(app.IPCHandler(), startup)
-	if err != nil {
-		log.Warnf("Failed to start Unix server: %v", err)
-	} else {
-		defer as.Close()
+	if !noIPC {
+		as, err := ipc.StartServer(app.IPCHandler(), startup)
+		if err != nil {
+			log.Warnf("Failed to start Unix server: %v", err)
+		} else {
+			defer as.Close()
+		}
 	}
 
 	// set the aerc version so that we can use it in the template funcs
 	templates.SetVersion(Version)
 
-	if retryExec {
-		// retry execution
-		err := ipc.ConnectAndExec(opts.Command)
-		if err != nil {
-			fmt.Fprintf(os.Stderr, "Failed to communicate to aerc: %v\n", err)
-			err = app.CloseBackends()
-			if err != nil {
-				log.Warnf("failed to close backends: %v", err)
-			}
+	endStartup := func() {
+		startupDone()
+		if len(opts.Command) == 0 {
 			return
 		}
-	}
-
-	if isatty.IsTerminal(os.Stderr.Fd()) {
-		setWindowTitle()
+		// Retry execution. Since IPC has already failed, we know no
+		// other aerc instance is running (or IPC was explicitly
+		// disabled); run the command directly.
+		err := app.Command(opts.Command)
+		if err != nil {
+			// no other aerc instance is running, so let
+			// this one stay running but show the error
+			errMsg := fmt.Sprintf("Startup command (%s) failed: %s\n",
+				strings.Join(opts.Command, " "), err)
+			log.Errorf(errMsg)
+			app.PushError(errMsg)
+		}
 	}
 
 	go func() {
@@ -284,7 +284,7 @@ loop:
 			// it will be ready. And in some cases, it may never be.
 			// At least, we can be confident that accepting IPC
 			// commands will not crash the whole process.
-			once.Do(startupDone)
+			once.Do(endStartup)
 		case callback := <-ui.Callbacks:
 			callback()
 		case <-ui.Redraw:
