@@ -7,7 +7,7 @@ import (
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/lib"
-	"git.sr.ht/~rjarry/aerc/log"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"github.com/emersion/go-imap"
 	"github.com/emersion/go-imap/client"
 )
@@ -57,6 +57,9 @@ func (w *IMAPWorker) connect() (*client.Client, error) {
 			}
 		}
 	case "imaps":
+		if w.config.insecure {
+			tlsConfig.InsecureSkipVerify = true
+		}
 		tlsConn := tls.Client(conn, tlsConfig)
 		c, err = client.New(tlsConn)
 		if err != nil {
@@ -98,8 +101,9 @@ func (w *IMAPWorker) connect() (*client.Client, error) {
 	if err := c.List("", "", info); err != nil {
 		return nil, fmt.Errorf("failed to retrieve delimiter: %w", err)
 	}
-	mailboxinfo := <-info
-	w.delimiter = mailboxinfo.Delimiter
+	if mailboxinfo := <-info; mailboxinfo != nil {
+		w.delimiter = mailboxinfo.Delimiter
+	}
 	if w.delimiter == "" {
 		// just in case some implementation does not follow standards
 		w.delimiter = "/"
@@ -123,19 +127,13 @@ func newTCPConn(addr string, timeout time.Duration) (*net.TCPConn, error) {
 	done := make(chan tcpConn)
 	go func() {
 		defer log.PanicHandler()
-		addr, err := net.ResolveTCPAddr("tcp", addr)
+
+		newConn, err := net.Dial("tcp", addr)
 		if err != nil {
 			done <- tcpConn{nil, err}
 			return
 		}
-
-		newConn, err := net.DialTCP("tcp", nil, addr)
-		if err != nil {
-			done <- tcpConn{nil, err}
-			return
-		}
-
-		done <- tcpConn{newConn, nil}
+		done <- tcpConn{newConn.(*net.TCPConn), nil}
 	}()
 
 	select {
