@@ -21,9 +21,9 @@ import (
 
 	aercLib "git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/iterator"
+	"git.sr.ht/~rjarry/aerc/lib/log"
 	"git.sr.ht/~rjarry/aerc/lib/watchers"
 	"git.sr.ht/~rjarry/aerc/lib/xdg"
-	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -575,7 +575,7 @@ func (w *Worker) handleFetchDirectoryThreaded(
 func (w *Worker) threads(ctx context.Context, uids []uint32,
 	criteria []*types.SortCriterion,
 ) ([]*types.Thread, error) {
-	builder := aercLib.NewThreadBuilder(iterator.NewFactory(false))
+	builder := aercLib.NewThreadBuilder(iterator.NewFactory(false), false)
 	msgInfos := make([]*models.MessageInfo, 0, len(uids))
 	mu := sync.Mutex{}
 	wg := sync.WaitGroup{}
